<?php
/**
 * Created by PhpStorm.
 * User: crose
 * Date: 1/2/16
 * Time: 10:57 PM
 */

namespace qfq;

use qfq;
use qfq\CodeException;

require_once(__DIR__ . '/../../qfq/Constants.php');

/**
 * Class Sanitize
 * @package qfq
 */
class Sanitize {


    private function __construct() {
        // Class should never be instantiated
    }

    /**
     * Check $value against given checkType/pattern. If check succeed, returns values.
     *   If check fails, depending on $mode, throws an UserException or return an empty string.
     *
     * @param string $value value to check
     * @param string $sanatizeClass SANITIZE_ALLOW_*
     * @param string $patternOrRange Pattern as regexp or MIN|MAX values
     * @param string $mode SANATIZE_EXCEPTION | SANATIZE_EMPTY_STRING
     * @return string
     * @throws UserException
     * @throws \qfq\CodeException
     */
    public static function sanitize($value, $sanatizeClass = SANITIZE_DEFAULT, $patternOrRange = '', $mode = SANATIZE_EMPTY_STRING) {
        $pattern = '';
        $minMax = array();
        $valueCompare = '';
        $errorText = "Value $value violates checkrule " . $sanatizeClass . " with pattern '$pattern'.";
        $errorCode = 0;

        // Prepare MIN|MAX
        switch ($sanatizeClass) {
            case SANITIZE_ALLOW_MIN_MAX:
                $minMax = explode('|', $patternOrRange);
                $valueCompare = $value;
                break;

            case SANITIZE_ALLOW_MIN_MAX_DATE:
                $minMax = explode('|', $patternOrRange);
                $valueCompare = Support::dateTime2mysql($value);
                $minMax[0] = Support::dateTime2mysql($minMax[0]);
                $minMax[1] = Support::dateTime2mysql($minMax[1]);
                break;

            default:
                break;
        }

        // Prepare Check
        switch ($sanatizeClass) {
            case SANITIZE_ALLOW_MIN_MAX:
            case SANITIZE_ALLOW_MIN_MAX_DATE:

                if ($minMax[0] === '' || $minMax[1] === '') {
                    throw new UserException('Missing definition of value for min or max.', ERROR_MISSING_MIN_MAX);
                }
            $errorText = "Value '$value' is out of range of '$patternOrRange'.";

                if ($minMax[0] <= $valueCompare && $valueCompare <= $minMax[1])
                    return $value;

                $errorCode = ERROR_MIN_MAX_VIOLATION;
                break;

            case SANITIZE_ALLOW_PATTERN:
                $pattern = $patternOrRange;
                break;

            case SANITIZE_ALLOW_DIGIT:
            case SANITIZE_ALLOW_EMAIL:
            case SANITIZE_ALLOW_ALNUMX:
                $arr = self::inputCheckPatternArray();
                $pattern = $arr[$sanatizeClass];
                break;

            case SANITIZE_ALLOW_ALL: // no checktype specified.
                return $value;

            default:
                throw new CodeException("Unknown checkType: " . $sanatizeClass, ERROR_UNKNOWN_CHECKTYPE);
        }

        // No error until here: do a final check
        if ($errorCode == 0) {
            if (preg_match("/$pattern/", $value) === 1)
                return $value;
            else
                $errorCode = ERROR_PATTERN_VIOLATION;
        }

        if ($mode === SANATIZE_EXCEPTION) {
            throw new UserException($errorText, $errorCode);
        }

        // check failed: return empty string
        return '';
    }

    /**
     * @return array
     */
    public static function inputCheckPatternArray() {
        return [
            SANITIZE_ALLOW_ALNUMX => '^(@|-|_|\.|,|;|:| |\/|\(|\)|[[:alnum:]])*$',
            SANITIZE_ALLOW_DIGIT => '^[\d]*$',
            SANITIZE_ALLOW_EMAIL => '^[_a-z0-9-]+(\.[_a-z0-9-]+)*@[a-z0-9-]+(\.[a-z0-9-]+)*(\.[a-z]{2,4})$',
            SANITIZE_ALLOW_MIN_MAX => '',
            SANITIZE_ALLOW_MIN_MAX_DATE => '',
            SANITIZE_ALLOW_PATTERN => '',
            SANITIZE_ALLOW_ALL => '.*'
        ];
    }

}